<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 02/28/2009 *
 * ===============                              Last change: 02/28/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : config-functions.php                             *
 * -------------------------------------------------------------------- *
 * Short description : Many non-MySQL functions (also file access)      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Viele Nicht-MySQL-Funktionen (auch Dateizugriff) *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Init the config array
function initConfig () {
	// Init not if already found
	if (isConfigLoaded()) {
		// Already initialized
		debug_report_bug(sprintf("[%s:%s] Configuration is already initialized.", __FUNCTION__, __LINE__));
	} // END - if

	// Set a minimum of configuration, required to by-pass some error triggers in getConfig()
	$GLOBALS['config'] = array(
		'code_length'         => 0,
		'patch_level'         => 0,
		'last_update'         => time(),
		'activate_xchange'    => 100,
		'enable_mod_title'    => 'Y',
		'update_filter_usage' => 'N',
		'ADMIN_REGISTERED'    => 'N',
		'MXCHANGE_INSTALLED'  => 'N',
		'DEFAULT_LANG'        => 'de',
		'DEFAULT_SALT_LENGTH' => 40,
		'DEBUG_MODE'          => 'N',
		'DEBUG_RESET'         => 'N',
		'DEBUG_MONTHLY'       => 'N',
		'DEBUG_WEEKLY'        => 'N',
		'DEBUG_REGEX'         => 'N',
		'ADMIN_REGISTERED'    => 'N',
		'sql_count'           => 0,
		'sql_time'            => 0,
		'num_templates'       => 0,
		'default_theme'       => 'default',
		'verbose_sql'         => 'Y',
		'def_refid'           => 0,
		'ENABLE_BACKLINK'     => 'Y',
		'display_debug_sqls'  => 'N',
		// Keep session_save_path to fall-back to php.ini setting
		'session_save_path'   => '',
		// For installation phase:
		'SMTP_HOSTNAME'       => '',
		'SMTP_USER'           => '',
		'SMTP_PASSWORD'       => '',
		'MT_WORD'             => '{--DEFAULT_MT_WORD--}',
	);
}

// Getter for $GLOBALS['config'] entries
function getConfig ($configEntry) {
	// Default value
	$value = null;

	// Is the entry there?
	if (!isset($GLOBALS['config'][$configEntry])) {
		// Raise an error of missing entries
		debug_report_bug(sprintf("[%s:%s] Configuration entry <em>%s</em> is missing.",
			__FUNCTION__,
			__LINE__,
			$configEntry
		));
	} // END - if

	// Return it
	return $GLOBALS['config'][$configEntry];
}

// Setter for $GLOBALS['config'] entries
function setConfigEntry ($configEntry, $value) {
	// Just set it (unsecured won't hurt?)
	$GLOBALS['config'][$configEntry] = $value;
}

// Checks wether the given config entry is set
function isConfigEntrySet ($configEntry) {
	return (isset($GLOBALS['config'][$configEntry]));
}

// Merges $GLOBALS['config'] with data in given array
function mergeConfig ($newConfig) {
	$GLOBALS['config'] = merge_array(getConfigArray(), $newConfig);
}

// Increment or init with given value or 1 as default the given config entry
function incrementConfigEntry ($configEntry, $value=1) {
	// Increment it if set or init it with 1
	if (getConfig($configEntry) > 0) {
		$GLOBALS['config'][$configEntry] += $value;
	} else {
		$GLOBALS['config'][$configEntry] = $value;
	}
}

// Checks wether the configuration array is set so the config is loaded
function isConfigLoaded () {
	// Check all
	return ((isset($GLOBALS['config'])) && (is_array($GLOBALS['config'])) && (count($GLOBALS['config']) > 0));
}

// Getter for whole $GLOBALS['config'] array
function getConfigArray () {
	// Default is null
	$return = null;

	// Is the config set?
	if (isConfigLoaded()) {
		// Then use it
		$return = $GLOBALS['config'];
	} // END - if

	// Return result
	return $return;
}

// Updates an old inc/config.php to a inc/cache/config-local.php file
function updateOldConfigFile () {
	// Watch out for these lines and execute them as single command
	// @TODO Make this all better... :-/
	$watchLines = array(
		'SITE_KEY'           => 'SITE_KEY',
		'DEFAULT_LANG'       => 'DEFAULT_LANG',
		'warn_no_pass'       => 'WARN_NO_PASS',
		'WRITE_FOOTER'       => 'WRITE_FOOTER',
		'OUTPUT_MODE'        => 'OUTPUT_MODE',
		'MAIN_TITLE'         => 'MAIN_TITLE',
		'SLOGAN'             => 'SLOGAN',
		'WEBMASTER'          => 'WEBMASTER',
		'mxchange_installed' => 'MXCHANGE_INSTALLED',
		'admin_registered'   => 'ADMIN_REGISTERED',
		'_MYSQL_PREFIX'      => '_MYSQL_PREFIX',
		'_TABLE_TYPE'        => '_TABLE_TYPE',
		'_DB_TYPE'           => '_DB_TYPE',
		'SMTP_HOSTNAME'      => 'SMTP_HOSTNAME',
		'SMTP_USER'           => 'SMTP_USER',
		'SMTP_PASSWORD'      => 'SMTP_PASSWORD',
		'ENABLE_BACKLINK'    => 'ENABLE_BACKLINK',
		'MAIN_TITLE'         => 'MAIN_TITLE',
		'SLOGAN'             => 'SLOGAN',
		'WEBMASTER'          => 'WEBMASTER',
		'PATH'               => 'PATH',
		'URL'                => 'URL',
	);

	// Make these lower-case! (damn stupid code...)
	$lowerCase = array('WARN_NO_PASS', 'MXCHANGE_INSTALLED', 'ADMIN_REGISTERED');

	// Special comments...
	$comments = array(
		'WARN_NO_PASS'       => 'NULLPASS-WARNING',
		'MXCHANGE_INSTALLED' => 'INSTALLED',
		'ADMIN_REGISTERED'   => 'ADMIN-SETUP',
		'_MYSQL_PREFIX'      => 'MYSQL-PREFIX',
		'_TABLE_TYPE'        => 'TABLE-TYPE',
		'_DB_TYPE'           => 'DATABASE-TYPE',
		'ENABLE_BACKLINK'    => 'BACKLINK',
		'host'               => 'MYSQL-HOST',
		'dbase'              => 'MYSQL-DBASE',
		'login'              => 'MYSQL-LOGIN',
		'password'           => 'MYSQL-PASSWORD'
	);

	// Copy template to new file destionation
	copyFileVerified(getConfig('PATH') . 'inc/config-local.php.dist', getConfig('CACHE_PATH') . 'config-local.php', 0644);

	// First of all, load the old one!
	$oldConfig = explode("\n", readFromFile(getConfig('PATH') . 'inc/config.php'));

	// Now, analyze every entry
	$done = array();
	foreach ($oldConfig as $line) {
		// Check all watch lines
		foreach ($watchLines as $old => $new) {
			// Add define() command around old one
			$old = "define('" . $old . "',";

			// Is the line found?
			if ((substr($line, 0, strlen($old)) == $old) && (!isset($done[$old]))) {
				// Entry found!
				//* DEBUG: */ outputHtml(htmlentities($line) . " - FOUND!<br />");

				// Eval the line...
				eval($line);

				// Setting config entry is new default behaviour!
				$function = 'setConfigEntry';

				// Default comment
				$comment = str_replace('_', '-', $new);

				// Do we have a special comment?
				if (isset($comments[$new])) {
					// Then use it
					$comment = $comments[$new];
				} // END - if

				// Do we need to make $new lowercase?
				$oldNew = $new;
				if (in_array($new, $lowerCase)) {
					// Then do so... :)
					$new = strtolower($new);
				} // END - if

				/// ... and write it to the new config
				//* DEBUG: */ outputHtml('function=' . $function . ',new=' . $new . ',comment=' . $comment . "<br />");
				changeDataInFile(getConfig('CACHE_PATH') . 'config-local.php', $comment, $function . "('" . $oldNew . "', \"", "\");", constant($new), 0);
				//* DEBUG: */ outputHtml("CHANGED!<br />");

				// Mark it as done
				$done[$old] = 1;
			} // END - if
		} // END - foreach
	} // END - foreach

	// By default the old array $MySQL was not found
	$found = false;

	// Analyze every entry again for the MySQL configuration
	foreach ($oldConfig as $line) {
		// Trim spaces
		$line = trim($line);

		// Is the $MySQL found?
		if (substr($line, 0, 6) == "\$MySQL") {
			// Okay found!
			$found = true;
		} elseif ($found === true) {
			// Now check this row
			if (substr($line, 0, 2) == ');') {
				// MySQL array is closed so stop looking for it
				break;
			} elseif (substr($line, 0, 2) == '//') {
				// Skip this line
				continue;
			}

			// Debug output only
			//* DEBUG: */ outputHtml(htmlentities($line) . " - MySQL!<br />");

			// Split parts so we can check them and prepare them
			$parts = explode('=>', $line);
			$key = substr(trim($parts[0]), 1, -1); $value = substr(trim($parts[1]), 1, -2);

			// We can now save the right part in new config file
			changeDataInFile(getConfig('CACHE_PATH') . 'config-local.php', $comments[$key], "	'".$key."'     => \"", "\",", $value, 0);
		}
	} // END - foreach

	// Finally remove old config file
	removeFile(getConfig('PATH') . 'inc/config.php');

	// Redirect to same URL to reload our new config
	redirectToUrl(getRequestUri());
}

// Update config entries
function updateConfiguration ($entries, $values, $updateMode='') {
	// Do not update config in CSS mode
	if ((getOutputMode() == 1) || (getOutputMode() == -1) || (isInstallationPhase())) {
		return;
	} // END - if

	// Do we have multiple entries?
	if (is_array($entries)) {
		// Walk through all
		$all = '';
		foreach ($entries as $idx => $entry) {
			// Update mode set?
			if (!empty($updateMode)) {
				// Update entry
				$all .= sprintf("`%s`=`%s`%s%s,", $entry, $entry, $updateMode, (float)$values[$idx]);
			} else {
				// Check if string or number
				if (($values[$idx] + 0) === $values[$idx]) {
					// Number detected
					$all .= sprintf("`%s`=%s,", $entry, (float)$values[$idx]);
				} elseif ($values[$idx] == 'UNIX_TIMESTAMP()') {
					// Function UNIX_TIMESTAMP() detected
					$all .= sprintf("`%s`=%s,", $entry, $values[$idx]);
				} else {
					// String detected
					$all .= sprintf("`%s`='%s',", $entry, SQL_ESCAPE($values[$idx]));
				}

				// Set it in config as well
				setConfigEntry($entry, $values[$idx]);
			}
		} // END - foreach

		// Remove last comma
		$entries = substr($all, 0, -1);
	} elseif (!empty($updateMode)) {
		// Update mode set
		// @TODO Find a way for updating configuration here
		$entries .= sprintf("=%s%s%s", $entries, $updateMode, (float)$values);
	} else {
		// Set it in config first
		setConfigEntry($entries, $values);

		// Regular entry to update
		$entries .= sprintf("='%s'", SQL_ESCAPE($values));
	}

	// Run database update
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, "entries={$entries}");
	SQL_QUERY("UPDATE `{?_MYSQL_PREFIX?}_config` SET ".$entries." WHERE `config`=0 LIMIT 1", __FUNCTION__, __LINE__);
	//* DEBUG: */ outputHtml(__FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):entries={$entries},affectedRows={$affectedRows}<br />");

	// Rebuild cache
	rebuildCacheFile('config', 'config');
}

// Filter for loading configuration
function FILTER_LOAD_CONFIGURATION ($no = 0) {
	// Check for cache extension, cache-array and if the requested configuration is in cache
	if ((isset($GLOBALS['cache_array']['config'][$no])) && (is_array($GLOBALS['cache_array']['config'][$no]))) {
		// Load config from cache
		//* DEBUG: */ outputHtml(gettype($GLOBALS['cache_array']['config'][$no])."<br />");
		mergeConfig($GLOBALS['cache_array']['config'][$no]);
		//foreach ($GLOBALS['cache_array']['config'][$no] as $key => $value) {
		//	setConfigEntry($key, $value);
		//} // END - foreach

		// Count cache hits if exists
		if ((isStatsEntrySet('cache_hits')) && (isExtensionActive('cache'))) {
			incrementStatsEntry('cache_hits');
		} // END - if
	} elseif ((!isExtensionActive('cache')) || (!isset($GLOBALS['cache_array']['config'][$no]))) {
		// Load config from DB
		$result_config = SQL_QUERY_ESC("SELECT * FROM `{?_MYSQL_PREFIX?}_config` WHERE config=%d LIMIT 1",
			array(bigintval($no)), __FUNCTION__, __LINE__);

		// Is the config there?
		if (SQL_NUMROWS($result_config) == 1) {
			// Get config from database
			mergeConfig(SQL_FETCHARRAY($result_config));
		} // END - if

		// Free result
		SQL_FREERESULT($result_config);

		// Remember this config in the array
		$GLOBALS['cache_array']['config'][$no] = getConfigArray();
	}
}

// [EOF]
?>
